@@ -12,11 +12,13 @@ extern "C" {
 #ifdef DEFINE_BUILTIN_GLOBALS
 #define DECLARE_BUILTIN(name) \
     JL_CALLABLE(jl_f_##name); \
-    jl_value_t *jl_builtin_##name
+    JL_DLLEXPORT jl_value_t *jl_builtin_##name; \
+    JL_DLLEXPORT jl_fptr_args_t jl_f_##name##_addr = &jl_f_##name
 #else
 #define DECLARE_BUILTIN(name) \
     JL_CALLABLE(jl_f_##name); \
-    extern jl_value_t *jl_builtin_##name
+    JL_DLLEXPORT extern jl_value_t *jl_builtin_##name; \
+    JL_DLLEXPORT extern jl_fptr_args_t jl_f_##name##_addr
 #endif
 
 DECLARE_BUILTIN(applicable);
@@ -27,6 +29,7 @@ DECLARE_BUILTIN(arrayref);
 DECLARE_BUILTIN(arrayset);
 DECLARE_BUILTIN(arraysize);
 DECLARE_BUILTIN(_call_in_world);
+DECLARE_BUILTIN(_call_in_world_total);
 DECLARE_BUILTIN(_call_latest);
 DECLARE_BUILTIN(replacefield);
 DECLARE_BUILTIN(const_arrayref);
@@ -51,13 +54,22 @@ DECLARE_BUILTIN(typeassert);
 DECLARE_BUILTIN(_typebody);
 DECLARE_BUILTIN(typeof);
 DECLARE_BUILTIN(_typevar);
+DECLARE_BUILTIN(donotdelete);
 
 JL_CALLABLE(jl_f_invoke_kwsorter);
+#ifdef DEFINE_BUILTIN_GLOBALS
+JL_DLLEXPORT jl_fptr_args_t jl_f_invoke_kwsorter_addr = &jl_f_invoke_kwsorter;
+#else
+JL_DLLEXPORT extern jl_fptr_args_t jl_f_invoke_kwsorter_addr;
+#endif
 JL_CALLABLE(jl_f__structtype);
 JL_CALLABLE(jl_f__abstracttype);
 JL_CALLABLE(jl_f__primitivetype);
 JL_CALLABLE(jl_f__setsuper);
 JL_CALLABLE(jl_f__equiv_typedef);
+JL_CALLABLE(jl_f_get_binding_type);
+JL_CALLABLE(jl_f_set_binding_type);
+JL_CALLABLE(jl_f_donotdelete);
 
 #ifdef __cplusplus
 }
