@@ -705,4 +705,10 @@ end
 # # https://github.com/JuliaLang/julia/pull/39845
 @test LinearAlgebra.LAPACK.liblapack == "libblastrampoline"
 
+# Issue #42762 https://github.com/JuliaLang/julia/issues/42762
+# Tests geqrf! and gerqf! with null column dimensions
+a = zeros(2,0), zeros(0)
+@test LinearAlgebra.LAPACK.geqrf!(a...) === a
+@test LinearAlgebra.LAPACK.gerqf!(a...) === a
+
 end # module TestLAPACK
